--------------------------------
-- Ivan Levashew aka OCTAGRAM --
--         Barnaul, 2009-2010 --
--------------------------------

-- Turbo_Private.Bits, 32bit version

with System, Interfaces, System.Storage_Elements;

package Turbo_Private.Bits is
   pragma Preelaborate;
   pragma Assert (System.Address'Size = 32, "Platform isn't 32bit, BITS must be readjusted");

   type Integer_Addr  is new Interfaces.Integer_32;
   type Unsigned_Addr is new Interfaces.Unsigned_32;

   function "+" (Item : Integer_Addr) return System.Storage_Elements.Storage_Offset;
   function "+" (Item : System.Storage_Elements.Storage_Offset) return Integer_Addr;

   -- Integer_Mem is Integer suitable for memory reference operations
   -- 4Gb is not a big value nowadays. 32bit systems should handle
   -- memory completely (e. g. operating 3Gb chunks is also an option)

   -- !!! Caution: offsets only
   -- On 64bit platforms Natural_Mem is not suitable for address storing
   subtype Integer_Mem is Interfaces.Integer_64
      range Interfaces.Integer_64'Val (-2**32) .. Interfaces.Integer_64'Val (2**32 - 1);
   subtype Natural_Mem is Integer_Mem range 0 .. Integer_Mem'Last;
   subtype Positive_Mem is Integer_Mem range 1 .. Integer_Mem'Last;

   -----------------
   -- Conversions --
   -----------------

   function "+" (Item : Unsigned_Addr) return System.Address;
   function "+" (Item : System.Address) return Unsigned_Addr;

   function "+" (Item : Unsigned_Addr) return System.Storage_Elements.Integer_Address;
   function "+" (Item : System.Storage_Elements.Integer_Address) return Unsigned_Addr;

   function "+" (Item : System.Address) return System.Storage_Elements.Integer_Address
      renames System.Storage_Elements.To_Integer;
   function "+" (Item : System.Storage_Elements.Integer_Address) return System.Address
      renames System.Storage_Elements.To_Address;

   -----------------
   -- Arythmetics --
   -----------------

   function "+" (Left : System.Address; Right : Integer_Addr)
     return System.Address;
   function "+" (Left : Integer_Addr; Right : System.Address)
     return System.Address;
   function "-" (Left : System.Address; Right : Integer_Addr)
     return System.Address;
   function "-" (Left, Right : System.Address)
     return Integer_Addr;
   function "mod" (Left : System.Address; Right : Integer_Addr)
     return Integer_Addr;

   -----------------

   function "+" (Left : System.Address; Right : Integer_Mem)
     return System.Address;
   function "+" (Left : Integer_Mem; Right : System.Address)
     return System.Address;
   function "-" (Left : System.Address; Right : Integer_Mem)
     return System.Address;
   function "-" (Left, Right : System.Address)
     return Integer_Mem;
   function "mod" (Left : System.Address; Right : Integer_Mem)
     return Integer_Mem;

   pragma Inline ("+", "-", "mod");

end Turbo_Private.Bits;
