--------------------------------
-- Ivan Levashew aka OCTAGRAM --
--              Barnaul, 2009 --
--------------------------------

-- Turbo_Private.Atomic, x86 version

-- TODO: CPUID
-- 486+ is expected currently
-- Does anybody program in Ada on 386? Let me know.

-- Several operations (e. g. Increment_Fetch) must be
-- rewritten using CAS on 386.

-- Implementation is partial.

-- TODO: Release/Aquire counterparts

-- Modern x86 CPUs has lfence, sfence, mfence insns.
-- I have noticed they are only used on x86_64 (but not on x86).
-- Is it possible to use "?fence" to implement Aquire/Release sematic?

-- TODO: Unsigned_32
-- TODO: _Addr, _64, _16, _8
-- TODO: Atomic_Boolean, Atomic_Address
-- TODO: Hide Memory_Barrier?


with System, Interfaces;

use System, Interfaces;

package Turbo_Private.Atomic is
   pragma Preelaborate;

   type Atomic_Integer_32 is limited private;

   type Atomic_Unsigned_32 is limited private;

   -- Improperly aligned 32bit values can actually be
   -- handled in an atomic way via cmpxchg8b
   -- I don't want to bother with these cases

-- TODO: unary "+"
-- variables need to be initialized somehow

   procedure Memory_Barrier;

   function Fetch (Source : in Atomic_Integer_32)
      return Integer_32;
   function Fetch (Source : not null access Atomic_Integer_32)
      return Integer_32;

   procedure Store (Target : not null access Atomic_Integer_32;
      Item : Integer_32);
   procedure Store (Target : in out Atomic_Integer_32;
      Item : Integer_32);
   function Exchange (Target : not null access Atomic_Integer_32;
      Item : Integer_32) return Integer_32;
   
   function Compare_Exchange (Target : not null access Atomic_Integer_32;
      Comparand, New_Item : Integer_32) return Boolean;
   function Compare_Exchange (Target : not null access Atomic_Integer_32;
      Comparand, New_Item : Integer_32) return Integer_32;
   function Compare_Exchange (Target : not null access Atomic_Integer_32;
      Comparand, New_Item : Integer_32; Fetched : not null access Integer_32)
      return Boolean;

   function Increment_Fetch (Addend : not null access Atomic_Integer_32)
      return Integer_32;
   function Fetch_Increment (Addend : not null access Atomic_Integer_32)
      return Integer_32;
   procedure Increment (Addend : in out Atomic_Integer_32);
   procedure Increment (Addend : not null access Atomic_Integer_32);

   function Increment_Fetch (Addend : not null access Atomic_Integer_32;
      Item : Integer_32) return Integer_32;
   function Fetch_Increment (Addend : not null access Atomic_Integer_32;
      Item : Integer_32) return Integer_32;
   procedure Increment (Addend : in out Atomic_Integer_32;
      Item : Integer_32);
   procedure Increment (Addend : not null access Atomic_Integer_32;
      Item : Integer_32);

   function Decrement_Fetch (Addend : not null access Atomic_Integer_32)
      return Integer_32;
   function Fetch_Decrement (Addend : not null access Atomic_Integer_32)
      return Integer_32;
   procedure Decrement (Addend : in out Atomic_Integer_32);
   procedure Decrement (Addend : not null access Atomic_Integer_32);

   function Decrement_Fetch (Addend : not null access Atomic_Integer_32;
      Item : Integer_32) return Integer_32;
   function Fetch_Decrement (Addend : not null access Atomic_Integer_32;
      Item : Integer_32) return Integer_32;
   procedure Decrement (Addend : in out Atomic_Integer_32;
      Item : Integer_32);
   procedure Decrement (Addend : not null access Atomic_Integer_32;
      Item : Integer_32);

private

   type Atomic_Integer_32 is record
      Item : aliased Integer_32;
      pragma Atomic (Item);
   end record;
   
   for Atomic_Integer_32'Size use 32;
   for Atomic_Integer_32'Alignment use 4;

   type Atomic_Unsigned_32 is record
      Item : aliased Unsigned_32;
      pragma Atomic (Item);
   end record;

   for Atomic_Unsigned_32'Size use 32;
   for Atomic_Unsigned_32'Alignment use 4;

end Turbo_Private.Atomic;
